<?php

namespace FiloBlu\AdyenExchangeRate\Console;

use FiloBlu\AdyenExchangeRate\Model\OrderProcessor;
use Magento\Framework\App\State;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Helper\ProgressBar;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

class FixOldOrder extends Command
{
    const INPUT_KEY_YEAR = 'year';

    const INPUT_TYPE_CURRENCY = 'currency';

    /**
     * @var OrderProcessor
     */
    private $orderProcessor;

    /**
     * @var State
     */
    private $state;

    public function __construct(
        OrderProcessor $orderProcessor,
        State $state
    )
    {
        $this->orderProcessor = $orderProcessor;
        $this->state = $state;
        parent::__construct();
    }

    protected function configure()
    {
        $this
            ->setName('filoblu:adyenexchangerate:fix-old-order')
            ->setDescription('Process orders and fix rate for specific year and currency. Retrieve rates if not present.')
            ->setDefinition($this->getInputList());
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        if ($input->getArgument(self::INPUT_KEY_YEAR)) {
            $requestedYear = trim($input->getArgument(self::INPUT_KEY_YEAR));
        }

        if($input->getArgument(self::INPUT_TYPE_CURRENCY)) {
            $currency = trim($input->getArgument(self::INPUT_TYPE_CURRENCY));
        }

        try {
            $this->state->getAreaCode();
        } catch (\Exception $e) {
            $this->state->setAreaCode('crontab');
        }

        $orders = $this->orderProcessor->getOrderByYearAndCurrency($requestedYear, $currency);
        if(!empty($orders)){
            $output->writeln('Found ' . count($orders) . ' to be processed.');
            $processed = array();
            $progress = new ProgressBar($output, count($orders));
            foreach ($orders as $order) {
                $this->orderProcessor->restoreCurrencyRate($order);
                $progress->advance(1);
                $processed[] = $order->getIncrementId();
            }
            $progress->finish();
            $output->writeln('');
            $output->writeln('The following orders were processed: ' . implode(', ', $processed));
        }
    }

    public function getInputList()
    {
        return [
            new InputArgument(
                self::INPUT_KEY_YEAR,
                InputArgument::REQUIRED,
                'Year of orders to be processed'
            ),
            new InputArgument(
                self::INPUT_TYPE_CURRENCY,
                InputArgument::REQUIRED,
                'Currency to process'
            )
        ];
    }
}